﻿using System;
using System.Threading;

public class ThreadPoolExample
{
    /// <summary>
    /// http://msdn.microsoft.com/en-us/library/3dasc8as(VS.80).aspx
    /// </summary>
    static void Main()
    {
        const int FibonacciCalculations = 20;
        const int MaxThreads = 3;

        // One event is used for each Fibonacci object
        ManualResetEvent[] doneEvents = new ManualResetEvent[FibonacciCalculations];
        Fibonacci[] fibArray = new Fibonacci[FibonacciCalculations];
        Random r = new Random();

        // Configure and launch threads using ThreadPool:
        Console.WriteLine("Launching {0} tasks...", FibonacciCalculations);
        Console.WriteLine();

        // Limit the number of concurrent threads
        ThreadPool.SetMaxThreads(MaxThreads, MaxThreads);

        // Start initialising threads...
        for (int i = 0; i < FibonacciCalculations; i++)
        {
            doneEvents[i] = new ManualResetEvent(false);
            Fibonacci f = new Fibonacci(r.Next(40, 50), doneEvents[i]);
            fibArray[i] = f;
            ThreadPool.QueueUserWorkItem(f.ThreadPoolCallback, i);
        }

        // Wait for all threads in pool to calculation...
        WaitHandle.WaitAll(doneEvents);
        Console.ForegroundColor = ConsoleColor.Green;

        Console.WriteLine();
        Console.WriteLine("All calculations are complete.");

        // Display the results...
        for (int i = 0; i < FibonacciCalculations; i++)
        {
            Fibonacci f = fibArray[i];
            Console.WriteLine("Fibonacci({0}) = {1}", f.N, f.FibOfN);
        }

        Console.WriteLine();
        Console.WriteLine("Total Execution Time: ");
        Console.Read();
    }
}

