using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace Mandelbrot
{
	public partial class MainForm : Form, ICallbackHandler
	{
		private static int nSections = 600;
		private double a0 = -2;
		private double b0 = -1.25;
		private double range = 2.5;
		private Bitmap bitmap = new Bitmap(nSections, nSections);

        ManualResetEvent[] doneEvents = new ManualResetEvent[nSections];
        const int MaxThreads = 8;

        private DateTime startTime;
        private DateTime endTime;

		public MainForm()
		{
			InitializeComponent();
		}

		public void Run()
        {
            ThreadPool.SetMaxThreads(MaxThreads, MaxThreads);
            //ThreadPool.SetMinThreads(MaxThreads, MaxThreads);

			double step = range/nSections;
			double a;
			double b;
			//Console.WriteLine("a0 = "+a0+"\tb0 = "+b0+"\trange = "+range);
			a = a0-step;

            lineCount = 0;
            startTime = endTime = DateTime.Now;       

            for (int j = 0; j < nSections; j++)
            {
                a += step;
                b = b0 - step;

                //doneEvents[j] = new ManualResetEvent(false);
                MandelbrotHandler mh = new MandelbrotHandler(this, nSections, j, a, b, step/*, doneEvents[j]*/);
                ThreadPool.QueueUserWorkItem(mh.Run, j);
                
            }

		}
		
        private int lineCount = 0;

        //Line update
		public void handle(Object res)
		{
            Line line = (Line)res;
            lock (this)
            {
                foreach (Point point in line)
                {
                    bitmap.SetPixel(point.x, point.y, point.color);
                    
                }
                lineCount++;
            }
            

            //Display image if all lines were painted
            if (lineCount == nSections)
            {
                Invalidate();

                endTime = DateTime.Now;
                TimeSpan diff = endTime - startTime;

                
                string message = String.Format("Finished in {0}s {1}ms", diff.Seconds, diff.Milliseconds);
                MessageBox.Show(message, "Time Capture", MessageBoxButtons.OK, MessageBoxIcon.Information);
            
            }
		}

		private void MainForm_Paint(object sender, PaintEventArgs e)
		{
            lock (this) {
                Graphics g = this.CreateGraphics();
                g.DrawImage(bitmap, 0, 0);
                g.Dispose();
            }
		}

		private void startToolStripMenuItem_Click(object sender, EventArgs e)
		{
			new Thread(this.Run).Start();
		}

		private void toolStripMenuItem2_Click(object sender, EventArgs e)
		{
			Close();
		}

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (bitmap != null)
            {
                SaveFileDialog sfd = new SaveFileDialog();
                sfd.DefaultExt = "bmp";
                sfd.Filter = "Bitmap Files (*.bmp)|*.bmp"; 

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    bitmap.Save(sfd.FileName);
                }
            }
        }
	}
}