using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;

namespace Adder
{
    public class AddNumbers
    {
        static Mutex mutex = new Mutex(false, "OnlyOneAtATime");

        static void Main() {
            int summe = 0;
            try
            {
                FileStream fs = new FileStream("daten.txt", FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
                StreamReader sr = new StreamReader(fs);
                while (true)
                {
                    mutex.WaitOne(); //Enter critical section

                    string line = string.Empty;

                    if ((line = sr.ReadLine()) == null)
                    {
                        mutex.ReleaseMutex();
                        break; //Exit loop if end of file reached   
                    }

                    try
                    {
                        summe += Int32.Parse(line);

                        Console.Clear();
                        Console.WriteLine("Addierer");
                        Console.WriteLine("Summe := " + summe);
                        Thread.Sleep(100); //Wait 100ms //Just for simulation
                    }
                    finally
                    {
                        mutex.ReleaseMutex(); //Exit critical section
                    }
                }
                sr.Close();
                fs.Close();
            }
            catch (AbandonedMutexException)
            {
                Console.WriteLine();
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("The exception that is thrown when one thread acquires a Mutex object"
                                + "that another thread has abandoned by exiting without releasing it.");
                Console.ForegroundColor = ConsoleColor.Gray;
            }
            catch (ApplicationException appex)
            {
                Console.WriteLine();
                Console.ForegroundColor = ConsoleColor.Red;
                Console.Write(appex.Message);
                Console.ForegroundColor = ConsoleColor.Gray;
            }
            catch (Exception ex)
            {
                Console.WriteLine();
                Console.ForegroundColor = ConsoleColor.Red;
                Console.Write(ex.StackTrace);
                Console.ForegroundColor = ConsoleColor.Gray;
            }

            Console.WriteLine();
            Console.WriteLine("Finished");
            Console.ReadLine();
        }
    }
}
